/** 
 *   FIXME ....
 *
 *   Survey for "antenna" position system.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ??? 
 **/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <malloc.h>
#include <sys/file.h>
#include <sys/stat.h>

#include "iar.h"
#include "antena.h"
#include "iar_bus.h"
#include "my_stdio.h"
#include "survey.h"
#include "encoders.h"
#include "coordinate.h"
#include "anemometer.h"

static int start_survey = 0;
static int fd_save = -1;
static char *file_save = NULL;
static int ra, dec;


static int open_save_file(char *buff)
{
	/* file_save */
	int fd_s;
	char data_dir[] = {"."};
	char *np = buff;
	char *endp;
	int alfa_norm, delta_norm;
	int grid = 5;
  	time_t now;
	struct tm *t;

	if( file_save == NULL)
		file_save = (char *) malloc(160);
	alfa_norm =grid_norm(strtod(np, &endp ), grid);
	np = endp + 1;
	delta_norm =grid_norm(strtod(np, &endp ), grid);
	/* Create date stamp and open data file */
	time(&now);
	t = localtime(&now);
	sprintf(file_save, "%s/%04d%02d%02d_%03d_%03d.dat", 
		data_dir, 
		t->tm_year + 1900, t->tm_mon+1, t->tm_mday,
		alfa_norm, delta_norm );
	//sprintf(file_save, "MM.dat");
	fd_s = open(file_save, O_WRONLY|O_CREAT|O_APPEND/*|O_SYNC*/,
		       S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (fd_s >= 0) {
		/* print header */
		fd_printf(fd_s,"# Antenna 2 position survey\n");
		fd_printf(fd_s,"# %04d/%02d/%02d %02d:%02d:%02d\n",
			  t->tm_year + 1900, t->tm_mon+1, t->tm_mday,
			  t->tm_hour, t->tm_min, t->tm_sec);
		fd_printf(fd_s,"# pos: %03d %03d\n#\n", 
			  alfa_norm, delta_norm );
	}

	return fd_s;
}


static int manager_track(int fd)
{
	char buff[160];
	int l;
	struct ib_packet pk;
	struct antena_packet *ap;
	struct encoders_data *d;

	ap = (struct antena_packet *)&pk.data;
	d = (struct encoders_data *)ap->data;
	ibus_read(fd, &pk);

	switch (ap->h.cmd) {
	case POSITION_ACM:
		l = sprintf(buff, "%8.4f %8.4f %3d\n", 
			ENC2DEG(d->val[ra]), 
			ENC2DEC(d->val[dec]),
			anemometer_conv(d->anemometer)
			);
		buff[l] = 0;
		//printf("manager_track: %s", buff); 
		if( start_survey ) {
			if (fd_save < 0) {
				/* init file */
				fd_save = open_save_file(buff);
				if (fd_save < 0) {
					perror("open save file:");
					antena_send_info(fd,  
							 "Error opening: %s", 
							 file_save);
					start_survey = 0;
				}else {
					antena_send_info(fd,  
							 "Save data in: %s", 
							 file_save);
				}
			}
			if ( fd_save > 0) 
				fd_printf(fd_save,"%s", buff);
		}
		break;
	case SURVEY_START_ACM:
		start_survey = 1;
		break;
	case SURVEY_STOP_ACM:
		start_survey = 0;
		if ( fd_save > 0) {
			antena_send_info(fd, "Close file: %s", file_save);
			start_survey = 0;
			close(fd_save);
			free(file_save);
			fd_save = -1;
		}
		break;
	case SURVEY_STATUS_ACM:
		l =  sizeof(struct antena_packet_head);
		if ( start_survey ) {
			l += sprintf(ap->data, "Save in file: %s", file_save);
		}else{
			l += sprintf(ap->data, "Stoped");
		}
		ap->h.cmd = INFO_ACM;
		ibus_send_data_to( fd, pk.h.addr, ap, l);
		break;
	default:
		antena_send_info(fd, "Survey: unknown cmd 0x%x", ap->h.cmd);
		return -1;
		break;
	}

	return 0;
}


int survey_main(int fd)
{
	fd_set rfds;
	struct timeval tv;
	int retval;
	int done;

	ra  = encoder_num[RA_CASE];
	dec = encoder_num[DEC_CASE];

	ibus_on( fd );
	done = 0;
	while( !done ) {
		FD_ZERO(&rfds);
		FD_SET(fd, &rfds);
		tv.tv_sec = 10;
		tv.tv_usec = 0;
		retval = select(fd + 1, &rfds, NULL, NULL, &tv);
		if (retval) {
			if (FD_ISSET(fd, &rfds)) {
				manager_track(fd);
			}
		}
	}
	ibus_remove( fd );

	return 0;
}
